--[[
		Object Class
		Copyright (c) 2012 Roland Yonaba
--]]

--[[
Copyright (c) 2012 Roland Yonaba

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.
--]]

local require = require
local assert = assert
local setmetatable = setmetatable

local vector = require '.Res.Libs.Physics.Vector'

local function checkVector(p) assert(p:isVector(),'Argument must be a vector') end
local vector_1 = vector(1,1)
local vector_0 = vector(0,0)

local Object = {}
Object.__index = Object

-- Creates a new physical object
local function new(position, velocity, acceleration)
	local object = { position = position or vector(),
					 velocity = velocity or vector(),
					 acceleration = acceleration or vector(),
					}
	return setmetatable(object,Object)
end

-- Sets physical object's position
function Object:setPosition (p)
	checkVector(p)
	self.position = p
end

-- Sets physical object's velocity
function Object:setVelocity(v)
	checkVector(v)
	self.velocity = v
end

-- Sets physical object's acceleration
function Object:setAcceleration(a)
	checkVector(a)
	self.acceleration = a
end

-- Gets physical object's position
function Object:getPosition()
	return self.position:clone()
end

-- Gets physical object's velocity
function Object:getVelocity()
	return self.velocity:clone()
end

-- Gets physical object's acceleration
function Object:getAcceleration()
	return self.acceleration:clone()
end

-- Integrate physics 
function Object:integrate(world)
	print('starting integration with dt',world.dt)
	local resultingAcceleration = (self:getAcceleration()+ world.gravity)
	print('resAcc',resultingAcceleration)
	local resultingVelocity = (resultingAcceleration*world.dt) + (self:getVelocity())
	print('resVel',resultingVelocity)
	local d = world.dt*world.damping
	d = vector_1 - d
	d = d:clamp(vector_0,vector_1)
	resultingVelocity = resultingVelocity * d
	print('resVel+world.damping',resultingVelocity)
	self:setVelocity(resultingVelocity)
	print('newVel',self:getVelocity())
	self:setPosition(self:getPosition() + (resultingVelocity*world.dt))
	print('newPosition',self:getPosition())
end

return setmetatable(Object, {__call = function (self,...) return new(...) end})