--[[
		Ball Class
		Copyright (c) 2012 Roland Yonaba
--]]

--[[
Copyright (c) 2012 Roland Yonaba

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgment in the product documentation would be
    appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
    distribution.
--]]

local require = require
local setmetatable = setmetatable
local assert = assert
local pi = math.pi

local vector = require '.Res.Libs.Physics.Vector'
local object = require '.Res.Libs.Physics.Object'

local function checkVector(p) assert(p:isVector(),'Argument must be a vector') end
local Ball = {}
Ball.__index = Ball

-- Creates a new Ball cat
local function new(position, velocity, acceleration, image)
	checkVector(position)
	checkVector(velocity)
	checkVector(acceleration)
	local ball = { 
					physics = object(position, velocity, acceleration),
					animAngle = 0,
					image = image,
					launched = false,
				  }
	return setmetatable(ball, Ball)
end

-- Resolves collision with the ground
function Ball:resolveGroundCollision(world)
	if (self.physics.position.y > world.ground.y) then
	local vel = self.physics:getVelocity()
	vel = vel:rotate(pi/2)
	vel = vel * vector(0.8,1)
	self.physics:setVelocity(vel)
	end
end

-- function Ball
-- Updates Ball cat physics
function Ball:update(world)
	if self.launched then
	self:resolveGroundCollision(world)
	self.physics:integrate(world)
	--self.animAngle = (self.animAngle + world.dt) % (2*pi)
	end
end

-- Draws Ball cat
function Ball:draw()
	love.graphics.draw(self.image,self.physics.position.x,self.physics.position.y,self.animAngle)
end

-- Prints Ball physics (debug)
function Ball:print(x,y,spacing)
	love.graphics.print(('Ball : x : %.2f y : %.2f'):format(self.physics.position.x,self.physics.position.y),x,y)
	love.graphics.print(('Ball : v_x : %.2f v_y : %.2f'):format(self.physics.velocity.x,self.physics.velocity.y),x,y+spacing)
	love.graphics.print(('Ball : a_x : %.2f a_y : %.2f'):format(self.physics.acceleration.x,self.physics.acceleration.y),x,y+spacing*2)
end
	return setmetatable(Ball, {__call = function (self,...) return new(...) end})
